"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Data = void 0;
const Root_1 = require("./Root");
const libs_1 = require("../libs");
const symbols_1 = require("../symbols");
const Omit = require("omit-empty");
const utils_1 = require("../utils");
class Data extends Root_1.Root {
    /**
     * ClassData constructor loads a initial state of the object, if needed.
     * @param initialData
     */
    constructor(initialData = {}, options = {}) {
        super();
        this.attributesDiscoveredInAllDataMode = [];
        this.options = libs_1._.defaults(options, { acceptAllData: false });
        this
            .assign(initialData)
            .resetDirtyState();
    }
    assign(initialData) {
        const deserialized = this.deserialize(initialData);
        Object.assign(this, deserialized);
        return this;
    }
    set(attributes) {
        const deserialized = this.deserialize(attributes);
        Object.assign(this, Omit(deserialized));
        return this;
    }
    /**
     * Loop through each data attribute of the class an run
     * a serialization function on the value, if provided
     */
    serialize() {
        this.debug('::serialize', 'airboat:classes:data');
        const result = {};
        symbols_1.initClassDataSymbols(this);
        this.getAttributeDefinitions().forEach(({ attrDef }) => {
            if (this[attrDef.name] !== undefined) {
                result[attrDef.name] = attrDef.serialize
                    ? attrDef.serialize(this[attrDef.name])
                    : this[attrDef.name];
            }
        });
        return Omit(result);
    }
    /**
     * Loop through each key of a given object, and will try to load
     * the values in the object if their keys is found in the data class
     * attribute list. If provided, will run a de-serialization function
     * on each value.
     *
     * @param object
     */
    deserialize(object) {
        this.debug('::deserialize', 'airboat:classes:data');
        const deserialized = {};
        symbols_1.initClassDataSymbols(this);
        this.getAttributeDefinitions(object).forEach(({ key, attrDef }) => {
            deserialized[key] = attrDef.deserialize
                ? attrDef.deserialize(object[key])
                : object[key];
        });
        return deserialized;
    }
    /**
     * Deep check if an attribute exists. Please refer to lodash for more details.
     *
     * @param key
     */
    has(key) {
        return libs_1._.has(this, key);
    }
    /**
     * Creates an object composed of the picked object properties.
     *
     * @param paths
     */
    pick(paths) {
        return libs_1._.pick(this.serialize(), paths);
    }
    /**
     * Gets the value at `path`. Returns `defaultValue` or false if nothing is found.
     *
     * @param path
     * @param defaultValue
     */
    get(path, defaultValue = false) {
        return (libs_1._.get(this, path, defaultValue));
    }
    getAndValidate(path, validationRule) {
        const value = this.get(path);
        const result = libs_1.Joi.validate(value, validationRule);
        utils_1.Assert(!result.error, utils_1.Exception.badRequest(result.error && result.error.message));
        return result.value;
    }
    /**
     * Checks if the value at `path` equals `value`
     *
     * @param path
     * @param value
     */
    is(path, value) {
        return this.get(path) === value;
    }
    /**
     * The opposite of `pick`; this method creates an object composed of the own and inherited enumerable property paths that are not omitted.
     *
     * @param paths
     */
    omit(paths) {
        return libs_1._.omit(this.serialize(), paths);
    }
    toJSON() {
        return this.serialize();
    }
    toString() {
        return JSON.stringify(this.serialize(), null, 2);
    }
    getAttributeDefinitions(data) {
        if (!this.options.acceptAllData) {
            return Object.keys(this[symbols_1.CLASS_DATA_ATTRIBUTES]).map((key) => ({
                key, attrDef: this[symbols_1.CLASS_DATA_ATTRIBUTES][key],
            }));
        }
        if (data) {
            this.attributesDiscoveredInAllDataMode = Object.keys(data);
        }
        return this.attributesDiscoveredInAllDataMode.map((key) => ({
            key, attrDef: { name: key },
        }));
    }
    resetDirtyState() {
        this.initialData = this.serialize();
        return this;
    }
    getDirtyAttributes() {
        const dirtyAttributes = [];
        const currentValues = this.serialize();
        this.getAttributeDefinitions().forEach(({ attrDef }) => {
            let initialValue = this.initialData[attrDef.name];
            let currentValue = currentValues[attrDef.name];
            if (initialValue && typeof initialValue.serialize === 'function') {
                initialValue = initialValue.serialize();
            }
            if (currentValue && typeof currentValue.serialize === 'function') {
                currentValue = currentValue.serialize();
            }
            if (!libs_1._.isEqual(initialValue, currentValue)) {
                dirtyAttributes.push(attrDef.name);
            }
        });
        return dirtyAttributes;
    }
}
exports.Data = Data;
//# sourceMappingURL=Data.js.map