"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Attribute = void 0;
const symbols_1 = require("./symbols");
const uuid_1 = require("uuid");
const crypto_js_1 = require("crypto-js");
const libs_1 = require("./libs");
const services_1 = require("./services");
const DEBUG_KEY = 'airboat:common:decorators';
// tslint:disable-next-line: function-name
function Attribute(attribute = {}) {
    return (target, attributeKey) => {
        services_1.Logger.debug(JSON.stringify(target.constructor.name), DEBUG_KEY);
        const attributeProps = libs_1._.has(attribute, 'isJoi')
            ? { validation: attribute }
            : attribute;
        symbols_1.initClassDataSymbols(target);
        attributeProps.name = attributeProps.name || attributeKey;
        target[symbols_1.CLASS_DATA_ATTRIBUTES][attributeKey] = attributeProps;
    };
}
exports.Attribute = Attribute;
Attribute.model = (modelClass, options = { optional: false }, attribute = {}) => {
    const validation = options.optional
        ? libs_1.Joi.object().model().optional()
        : libs_1.Joi.object().model().required();
    return Attribute(libs_1._.defaults(attribute, {
        validation,
        deserialize: (o = {}) => new modelClass(o),
        serialize: (o) => o.serialize(),
    }));
};
Attribute.modelList = (model, options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.array().items(libs_1.Joi.object().model()).required(),
        deserialize: (list = []) => list.map((a) => new model(a)),
        serialize: (list) => list.map((a) => a.serialize()),
    }));
};
Attribute.dateTime = (options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, {
        optional: false,
    });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.date().optional()
            : libs_1.Joi.date().default(() => new Date(), 'now').optional(),
        deserialize: (o) => o ? new Date(o) : undefined,
        serialize: (o) => o.toISOString(),
    }));
};
Attribute.uuid = (options = {}, attribute = {}) => {
    const { required } = libs_1._.defaults(options, { required: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: required
            ? libs_1.Joi.string().guid().required()
            : libs_1.Joi.string().guid().default(uuid_1.v4, 'new uuid').optional(),
    }));
};
Attribute.workspaceId = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().guid().required(),
    }));
};
Attribute.key = (options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, { optional: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.string().max(50).regex(new RegExp('^([a-z]*)(-[a-z]+)*$')).optional()
            : libs_1.Joi.string().max(50).regex(new RegExp('^([a-z]*)(-[a-z]+)*$')).required(),
    }));
};
Attribute.keyDot = (options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, { optional: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.string().max(50).regex(new RegExp('^([a-z]*)(\.[a-z]+)*$')).optional()
            : libs_1.Joi.string().max(50).regex(new RegExp('^([a-z]*)(\.[a-z]+)*$')).required(),
    }));
};
Attribute.string = (options = {}, attribute = {}) => {
    const { optional, max } = libs_1._.defaults(options, { optional: false, max: 250 });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.string().max(max).optional()
            : libs_1.Joi.string().max(max).required()
    }));
};
Attribute.email = (options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, { optional: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.string().email().max(255).optional()
            : libs_1.Joi.string().email().max(255).required()
    }));
};
Attribute.uri = (options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, { optional: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.string().uri().max(255).optional()
            : libs_1.Joi.string().uri().max(255).required()
    }));
};
Attribute.oneOf = (validOptions, options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, { optional: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.any().valid(validOptions).optional()
            : libs_1.Joi.any().valid(validOptions).required(),
    }));
};
Attribute.hash = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().max(500).required(),
    }));
};
Attribute.boolean = (options = {}, attribute = {}) => {
    const { optional } = libs_1._.defaults(options, { optional: false });
    return Attribute(libs_1._.defaults(attribute, {
        validation: optional
            ? libs_1.Joi.bool().optional()
            : libs_1.Joi.bool().required(),
    }));
};
Attribute.hash = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().max(500).required(),
    }));
};
Attribute.encryptedObject = ({ secret }, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.object().max(10).required(),
        deserialize: (v) => {
            if (!v) {
                return undefined;
            }
            try {
                return JSON.parse(crypto_js_1.Rabbit.decrypt(v, secret).toString(crypto_js_1.enc.Utf8));
            }
            catch (e) {
                return v;
            }
        },
        serialize: (v) => {
            if (!v) {
                return undefined;
            }
            return crypto_js_1.Rabbit.encrypt(JSON.stringify(v), secret).toString();
        },
    }));
};
//# sourceMappingURL=decorators.js.map